import { __decorate, __param } from 'vscode/external/tslib/tslib.es6.js';
import { BroadcastDataChannel } from 'vscode/vscode/vs/base/browser/broadcast';
import { revive } from 'vscode/vscode/vs/base/common/marshalling';
import { IEnvironmentService } from 'vscode/vscode/vs/platform/environment/common/environment.service';
import { IFileService } from 'vscode/vscode/vs/platform/files/common/files.service';
import { ILogService } from 'vscode/vscode/vs/platform/log/common/log.service';
import { IUriIdentityService } from 'vscode/vscode/vs/platform/uriIdentity/common/uriIdentity.service';
import { UserDataProfilesService, reviveProfile } from 'vscode/vscode/vs/platform/userDataProfile/common/userDataProfile';

let BrowserUserDataProfilesService = class BrowserUserDataProfilesService extends UserDataProfilesService {
    constructor(environmentService, fileService, uriIdentityService, logService) {
        super(environmentService, fileService, uriIdentityService, logService);
        this.changesBroadcastChannel = this._register(( new BroadcastDataChannel(`${UserDataProfilesService.PROFILES_KEY}.changes`)));
        this._register(this.changesBroadcastChannel.onDidReceiveData(changes => {
            try {
                this._profilesObject = undefined;
                const added = ( changes.added.map(p => reviveProfile(p, this.profilesHome.scheme)));
                const removed = ( changes.removed.map(p => reviveProfile(p, this.profilesHome.scheme)));
                const updated = ( changes.updated.map(p => reviveProfile(p, this.profilesHome.scheme)));
                this.updateTransientProfiles(added.filter(a => a.isTransient), removed.filter(a => a.isTransient), updated.filter(a => a.isTransient));
                this._onDidChangeProfiles.fire({
                    added,
                    removed,
                    updated,
                    all: this.profiles
                });
            }
            catch (error) {  }
        }));
    }
    updateTransientProfiles(added, removed, updated) {
        if (added.length) {
            this.transientProfilesObject.profiles.push(...added);
        }
        if (removed.length || updated.length) {
            const allTransientProfiles = this.transientProfilesObject.profiles;
            this.transientProfilesObject.profiles = [];
            for (const profile of allTransientProfiles) {
                if (( removed.some(p => profile.id === p.id))) {
                    continue;
                }
                this.transientProfilesObject.profiles.push(updated.find(p => profile.id === p.id) ?? profile);
            }
        }
    }
    getStoredProfiles() {
        try {
            const value = localStorage.getItem(UserDataProfilesService.PROFILES_KEY);
            if (value) {
                return revive(JSON.parse(value));
            }
        }
        catch (error) {
            this.logService.error(error);
        }
        return [];
    }
    triggerProfilesChanges(added, removed, updated) {
        super.triggerProfilesChanges(added, removed, updated);
        this.changesBroadcastChannel.postData({ added, removed, updated });
    }
    saveStoredProfiles(storedProfiles) {
        localStorage.setItem(UserDataProfilesService.PROFILES_KEY, JSON.stringify(storedProfiles));
    }
    getStoredProfileAssociations() {
        const migrateKey = 'profileAssociationsMigration';
        try {
            const value = localStorage.getItem(UserDataProfilesService.PROFILE_ASSOCIATIONS_KEY);
            if (value) {
                let associations = JSON.parse(value);
                if (!localStorage.getItem(migrateKey)) {
                    associations = this.migrateStoredProfileAssociations(associations);
                    this.saveStoredProfileAssociations(associations);
                    localStorage.setItem(migrateKey, 'true');
                }
                return associations;
            }
        }
        catch (error) {
            this.logService.error(error);
        }
        return {};
    }
    saveStoredProfileAssociations(storedProfileAssociations) {
        localStorage.setItem(UserDataProfilesService.PROFILE_ASSOCIATIONS_KEY, JSON.stringify(storedProfileAssociations));
    }
};
BrowserUserDataProfilesService = ( __decorate([
    ( __param(0, IEnvironmentService)),
    ( __param(1, IFileService)),
    ( __param(2, IUriIdentityService)),
    ( __param(3, ILogService))
], BrowserUserDataProfilesService));

export { BrowserUserDataProfilesService };
