import { __decorate, __param } from 'vscode/external/tslib/tslib.es6.js';
import { TreeItemCollapsibleState } from 'vscode/vscode/vs/workbench/common/views';
import { IViewDescriptorService } from 'vscode/vscode/vs/workbench/common/views.service';
import { localizeWithPath } from 'vscode/vscode/vs/nls';
import { TreeViewPane } from 'vscode/vscode/vs/workbench/browser/parts/views/treeView';
import { IInstantiationService } from 'vscode/vscode/vs/platform/instantiation/common/instantiation';
import { IUserDataSyncService, IUserDataSyncEnablementService } from 'vscode/vscode/vs/platform/userDataSync/common/userDataSync.service';
import { registerAction2, Action2, MenuId } from 'vscode/vscode/vs/platform/actions/common/actions';
import { ContextKeyExpr } from 'vscode/vscode/vs/platform/contextkey/common/contextkey';
import { IContextKeyService } from 'vscode/vscode/vs/platform/contextkey/common/contextkey.service';
import { URI } from 'vscode/vscode/vs/base/common/uri';
import { IEditorService } from 'vscode/vscode/vs/workbench/services/editor/common/editorService.service';
import { getSyncAreaLabel, SYNC_CONFLICTS_VIEW_ID } from 'vscode/vscode/vs/workbench/services/userDataSync/common/userDataSync';
import { IUserDataSyncWorkbenchService } from 'vscode/vscode/vs/workbench/services/userDataSync/common/userDataSync.service';
import { basename, isEqual } from 'vscode/vscode/vs/base/common/resources';
import { append, $ } from 'vscode/vscode/vs/base/browser/dom';
import { IKeybindingService } from 'vscode/vscode/vs/platform/keybinding/common/keybinding.service';
import { IContextMenuService } from 'vscode/vscode/vs/platform/contextview/browser/contextView.service';
import { IConfigurationService } from 'vscode/vscode/vs/platform/configuration/common/configuration.service';
import { IOpenerService } from 'vscode/vscode/vs/platform/opener/common/opener.service';
import { IThemeService } from 'vscode/vscode/vs/platform/theme/common/themeService.service';
import { ITelemetryService } from 'vscode/vscode/vs/platform/telemetry/common/telemetry.service';
import { INotificationService } from 'vscode/vscode/vs/platform/notification/common/notification.service';
import { Codicon } from 'vscode/vscode/vs/base/common/codicons';
import { reviveProfile } from 'vscode/vscode/vs/platform/userDataProfile/common/userDataProfile';
import { IUserDataProfilesService } from 'vscode/vscode/vs/platform/userDataProfile/common/userDataProfile.service';
import { DEFAULT_EDITOR_ASSOCIATION } from 'vscode/vscode/vs/workbench/common/editor';

let UserDataSyncConflictsViewPane = class UserDataSyncConflictsViewPane extends TreeViewPane {
    constructor(options, editorService, keybindingService, contextMenuService, configurationService, contextKeyService, viewDescriptorService, instantiationService, openerService, themeService, telemetryService, notificationService, userDataSyncService, userDataSyncWorkbenchService, userDataSyncEnablementService, userDataProfilesService) {
        super(options, keybindingService, contextMenuService, configurationService, contextKeyService, viewDescriptorService, instantiationService, openerService, themeService, telemetryService, notificationService);
        this.editorService = editorService;
        this.userDataSyncService = userDataSyncService;
        this.userDataSyncWorkbenchService = userDataSyncWorkbenchService;
        this.userDataSyncEnablementService = userDataSyncEnablementService;
        this.userDataProfilesService = userDataProfilesService;
        this._register(this.userDataSyncService.onDidChangeConflicts(() => this.treeView.refresh()));
        this.registerActions();
    }
    renderTreeView(container) {
        super.renderTreeView(append(container, $('')));
        const that = this;
        this.treeView.message = ( localizeWithPath(
            'vs/workbench/contrib/userDataSync/browser/userDataSyncConflictsView',
            'explanation',
            "Please go through each entry and merge to resolve conflicts."
        ));
        this.treeView.dataProvider = { getChildren() { return that.getTreeItems(); } };
    }
    async getTreeItems() {
        const roots = [];
        const conflictResources = ( this.userDataSyncService.conflicts
            .map(conflict => ( conflict.conflicts.map(
            resourcePreview => ({ ...resourcePreview, syncResource: conflict.syncResource, profile: conflict.profile })
        ))))
            .flat()
            .sort((a, b) => a.profile.id === b.profile.id ? 0 : a.profile.isDefault ? -1 : b.profile.isDefault ? 1 : a.profile.name.localeCompare(b.profile.name));
        const conflictResourcesByProfile = [];
        for (const previewResource of conflictResources) {
            let result = conflictResourcesByProfile[conflictResourcesByProfile.length - 1]?.[0].id === previewResource.profile.id ? conflictResourcesByProfile[conflictResourcesByProfile.length - 1][1] : undefined;
            if (!result) {
                conflictResourcesByProfile.push([previewResource.profile, result = []]);
            }
            result.push(previewResource);
        }
        for (const [profile, resources] of conflictResourcesByProfile) {
            const children = [];
            for (const resource of resources) {
                const handle = JSON.stringify(resource);
                const treeItem = {
                    handle,
                    resourceUri: resource.remoteResource,
                    label: { label: basename(resource.remoteResource), strikethrough: resource.mergeState === "accepted"  && ((resource.localChange === 3  || resource.remoteChange === 3) ) },
                    description: getSyncAreaLabel(resource.syncResource),
                    collapsibleState: TreeItemCollapsibleState.None,
                    command: { id: `workbench.actions.sync.openConflicts`, title: '', arguments: [{ $treeViewId: '', $treeItemHandle: handle }] },
                    contextValue: `sync-conflict-resource`
                };
                children.push(treeItem);
            }
            roots.push({
                handle: profile.id,
                label: { label: profile.name },
                collapsibleState: TreeItemCollapsibleState.Expanded,
                children
            });
        }
        return conflictResourcesByProfile.length === 1 && conflictResourcesByProfile[0][0].isDefault ? roots[0].children ?? [] : roots;
    }
    parseHandle(handle) {
        const parsed = JSON.parse(handle);
        return {
            syncResource: parsed.syncResource,
            profile: reviveProfile(parsed.profile, this.userDataProfilesService.profilesHome.scheme),
            localResource: URI.revive(parsed.localResource),
            remoteResource: URI.revive(parsed.remoteResource),
            baseResource: URI.revive(parsed.baseResource),
            previewResource: URI.revive(parsed.previewResource),
            acceptedResource: URI.revive(parsed.acceptedResource),
            localChange: parsed.localChange,
            remoteChange: parsed.remoteChange,
            mergeState: parsed.mergeState,
        };
    }
    registerActions() {
        const that = this;
        this._register(registerAction2(class OpenConflictsAction extends Action2 {
            constructor() {
                super({
                    id: `workbench.actions.sync.openConflicts`,
                    title: ( localizeWithPath(
                        'vs/workbench/contrib/userDataSync/browser/userDataSyncConflictsView',
                        { key: 'workbench.actions.sync.openConflicts', comment: ['This is an action title to show the conflicts between local and remote version of resources'] },
                        "Show Conflicts"
                    )),
                });
            }
            async run(accessor, handle) {
                const conflict = that.parseHandle(handle.$treeItemHandle);
                return that.open(conflict);
            }
        }));
        this._register(registerAction2(class AcceptRemoteAction extends Action2 {
            constructor() {
                super({
                    id: `workbench.actions.sync.acceptRemote`,
                    title: ( localizeWithPath(
                        'vs/workbench/contrib/userDataSync/browser/userDataSyncConflictsView',
                        'workbench.actions.sync.acceptRemote',
                        "Accept Remote"
                    )),
                    icon: Codicon.cloudDownload,
                    menu: {
                        id: MenuId.ViewItemContext,
                        when: ( ContextKeyExpr.and(( ContextKeyExpr.equals('view', SYNC_CONFLICTS_VIEW_ID)), ( ContextKeyExpr.equals('viewItem', 'sync-conflict-resource')))),
                        group: 'inline',
                        order: 1,
                    },
                });
            }
            async run(accessor, handle) {
                const conflict = that.parseHandle(handle.$treeItemHandle);
                await that.userDataSyncWorkbenchService.accept({ syncResource: conflict.syncResource, profile: conflict.profile }, conflict.remoteResource, undefined, that.userDataSyncEnablementService.isEnabled());
            }
        }));
        this._register(registerAction2(class AcceptLocalAction extends Action2 {
            constructor() {
                super({
                    id: `workbench.actions.sync.acceptLocal`,
                    title: ( localizeWithPath(
                        'vs/workbench/contrib/userDataSync/browser/userDataSyncConflictsView',
                        'workbench.actions.sync.acceptLocal',
                        "Accept Local"
                    )),
                    icon: Codicon.cloudUpload,
                    menu: {
                        id: MenuId.ViewItemContext,
                        when: ( ContextKeyExpr.and(( ContextKeyExpr.equals('view', SYNC_CONFLICTS_VIEW_ID)), ( ContextKeyExpr.equals('viewItem', 'sync-conflict-resource')))),
                        group: 'inline',
                        order: 2,
                    },
                });
            }
            async run(accessor, handle) {
                const conflict = that.parseHandle(handle.$treeItemHandle);
                await that.userDataSyncWorkbenchService.accept({ syncResource: conflict.syncResource, profile: conflict.profile }, conflict.localResource, undefined, that.userDataSyncEnablementService.isEnabled());
            }
        }));
    }
    async open(conflictToOpen) {
        if (!( this.userDataSyncService.conflicts.some(({ conflicts }) => ( conflicts.some(
            ({ localResource }) => isEqual(localResource, conflictToOpen.localResource)
        ))))) {
            return;
        }
        const remoteResourceName = ( localizeWithPath(
            'vs/workbench/contrib/userDataSync/browser/userDataSyncConflictsView',
            { key: 'remoteResourceName', comment: ['remote as in file in cloud'] },
            "{0} (Remote)",
            basename(conflictToOpen.remoteResource)
        ));
        const localResourceName = ( localizeWithPath(
            'vs/workbench/contrib/userDataSync/browser/userDataSyncConflictsView',
            'localResourceName',
            "{0} (Local)",
            basename(conflictToOpen.remoteResource)
        ));
        await this.editorService.openEditor({
            input1: { resource: conflictToOpen.remoteResource, label: ( localizeWithPath(
                'vs/workbench/contrib/userDataSync/browser/userDataSyncConflictsView',
                'Theirs',
                'Theirs'
            )), description: remoteResourceName },
            input2: { resource: conflictToOpen.localResource, label: ( localizeWithPath(
                'vs/workbench/contrib/userDataSync/browser/userDataSyncConflictsView',
                'Yours',
                'Yours'
            )), description: localResourceName },
            base: { resource: conflictToOpen.baseResource },
            result: { resource: conflictToOpen.previewResource },
            options: {
                preserveFocus: true,
                revealIfVisible: true,
                pinned: true,
                override: DEFAULT_EDITOR_ASSOCIATION.id
            }
        });
        return;
    }
};
UserDataSyncConflictsViewPane = ( __decorate([
    ( __param(1, IEditorService)),
    ( __param(2, IKeybindingService)),
    ( __param(3, IContextMenuService)),
    ( __param(4, IConfigurationService)),
    ( __param(5, IContextKeyService)),
    ( __param(6, IViewDescriptorService)),
    ( __param(7, IInstantiationService)),
    ( __param(8, IOpenerService)),
    ( __param(9, IThemeService)),
    ( __param(10, ITelemetryService)),
    ( __param(11, INotificationService)),
    ( __param(12, IUserDataSyncService)),
    ( __param(13, IUserDataSyncWorkbenchService)),
    ( __param(14, IUserDataSyncEnablementService)),
    ( __param(15, IUserDataProfilesService))
], UserDataSyncConflictsViewPane));

export { UserDataSyncConflictsViewPane };
